#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include "intlist.h"
#include "sequence.h"


static char *
Sequence_read_name (FILE *fp) {
  char *name_char;
  int n = 0, i, c;
  Intlist_T namelist = (Intlist_T) NULL;

  while (!feof(fp) && ((c = fgetc(fp)) != '\n')) {
    namelist = (Intlist_T) Intlist_push(c,namelist);
    ++n;
  }
  if (n == 0) {
    return (char *) NULL;
  } else if (feof(fp)) {
    fprintf(stderr,"File ended while reading name of sequence.\n");
    return (char *) NULL;
  } else {
    name_char = (char *) calloc(n+1,sizeof(char));
    name_char[n] = '\0';
    for (i = n-1; i >= 0; --i) {
      namelist = Intlist_pop(namelist,&c);
      name_char[i] = (char) c;
    }
    return name_char;
  }
}


static char *
Sequence_read_seq (FILE *fp, int *length) {
  char *sequence_char;
  int n = 0, i, c;
  Intlist_T seqlist = (Intlist_T) NULL;

  while (!feof(fp) && ((c = fgetc(fp)) != '>')) {
    if (isalpha(c)) {
      seqlist = (Intlist_T) Intlist_push(c,seqlist);
      ++n;
    }
  }
  if (c == '>') {
    ungetc(c,fp);
  }
  *length = n;

  if (*length == 0) {
    return (char *) NULL;
  } else {
    sequence_char = (char *) calloc(n,sizeof(char));
    for (i = n-1; i >= 0; --i) {
      seqlist = Intlist_pop(seqlist,&c);
      sequence_char[i] = (char) c;
    }
    return sequence_char;
  }  
}

/* Returns NULL if there is no sequence is read. */
char *
Sequence_read (FILE *fp, char **name, int *length) {
  int c;

  if (feof(fp)) {
    return (char *) NULL;
  } else {
    if ((c = fgetc(fp)) == '>') {
      *name = Sequence_read_name(fp);
      if (*name == NULL) {
	return (char *) NULL;
      } else {
	return Sequence_read_seq(fp,length);
      }
    } else {
      ungetc(c,fp);
      *name = (char *) NULL;
      return Sequence_read_seq(fp,length);
    }
  }
}

