#include "assert.h"
#include "dist.h"
#include <math.h>
#include <stdio.h>
#include <stdlib.h>



#define T Dist_T
struct T {
  int min_score;
  int max_score;
  int length;
  double *contents;
};


T
Dist_init () {
  T dist = (T) malloc(sizeof(*dist));
  
  dist->min_score = 0;
  dist->max_score = 0;
  dist->length = 1;
  dist->contents = (double *) calloc(dist->length,sizeof(double));
  dist->contents[0] = 1.0;
  return dist;
}



T
Dist_new (int min_score, int max_score) {
  T dist = (T) malloc(sizeof(*dist));
  
  dist->min_score = min_score;
  dist->max_score = max_score;
  dist->length = max_score - min_score + 1;
  dist->contents = (double *) calloc(dist->length,sizeof(double));
  return dist;
}

void
Dist_free (T *dist) {
  free((*dist)->contents);
  free(*dist);
}

int
Dist_min_score (T dist) {
  return dist->min_score;
}

int
Dist_max_score (T dist) {
  return dist->max_score;
}


double
Dist_get_prob (T dist, int score) {
  assert(score >= dist->min_score);
  assert(score <= dist->max_score);
  return dist->contents[score - dist->min_score];
}

void
Dist_add_prob (T dist, int score, double prob) {
  assert(score >= dist->min_score);
  assert(score <= dist->max_score);
  dist->contents[score - dist->min_score] += prob;
}


void
Dist_add_dist (T new, T dist) {
  int i, score;

  for (i = 0; i < dist->length; ++i) {
    score = dist->min_score + i;
    new->contents[score - new->min_score] += dist->contents[i];
  }
}


static double
report_prob (T dist, int score) {
  return dist->contents[score - dist->min_score];
}


void
Dist_print (T dist) {
  int i;

  for (i = 0; i < dist->length; ++i) {
    if (dist->contents[i] != 0.0) {
      printf("%d:%f ",dist->min_score+i,dist->contents[i]);
    }
  }
  printf("\n");
}


void
Dist_print_steps (T dist, int fromsig, int tosig) {
  int min_score, max_score, i, sig;
  double mostsig, prob, G = 0.0;

  min_score = dist->min_score;
  max_score = dist->max_score;

  mostsig = -log10(report_prob(dist,max_score));
  for (sig = fromsig; (sig >= mostsig) && (sig >= tosig); --sig) {
    printf("%d ",max_score+1);
  }
  for (i = dist->length-1; (i >= 0) && (sig >= tosig); --i) {
    prob = report_prob(dist,min_score+i);
    if (prob > 0.0) {
      G += prob;
      if (-log10(G) <= sig) {
	printf("%d ",min_score+i);
	--sig;
      }
    }
  }
  for ( ; sig >= tosig; --sig) {
    printf("%d ",min_score+i);
  }
  printf("\n");
}

void
Dist_print_probs (T dist, bool continuous) {
  int i;
  double prob, G = 0.0;

  for (i = dist->length-1; i >= 0; --i) {
    prob = report_prob(dist,dist->min_score+i);
    if (continuous || (prob > 0.0)) {
      G += prob;
      printf("%d %f %e %e\n",dist->min_score+i,log10(G),G,prob);
    }
  }
}

void
Dist_print_threshold (T dist, double p_threshold, bool continuous) {
  int i;
  double prob, G = 0.0;

  for (i = dist->length-1; i >= 0; --i) {
    prob = report_prob(dist,dist->min_score+i);
    if (continuous || (prob > 0.0)) {
      G += prob;
      if (G > p_threshold) {
	printf("%d %f %e %e\n",dist->min_score+i,log10(G),G,prob);
	return;
      }
    }
  }
}

double
Dist_check (T dist) {
  int i;
  double prob = 0.0;

  for (i = 0; i < dist->length; i++) {
    prob += dist->contents[i];
  }
  return prob;
}

