#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "assert.h"
#include "alphabet.h"
#include "prob.h"
#include "freq-data.h"
#include "frequencies.h"

#define T Frequencies_T
struct T {
  int maxorder;
  int numentries;
  int *lengths;
  double *contents;
};

T
Frequencies_new (int maxorder) {
  T frequencies = (T) malloc(sizeof(*frequencies));
  int length = 1, i, numentries = 0;

  frequencies->maxorder = maxorder;
  frequencies->lengths = (int *) calloc(maxorder+1,sizeof(int));
  for (i = 0; i <= maxorder; ++i) {
    length *= ALPHABET_SIZE;
    frequencies->lengths[i] = length;
    numentries += length;
  }
  frequencies->numentries = numentries;
  frequencies->contents = (double *) calloc(numentries,sizeof(double));
  return frequencies;
}

T
Frequencies_read (FILE *fp, int maxorder) {
  T frequencies;
  double freq;
  int i;

  frequencies = Frequencies_new(maxorder);
  for (i = 0; i < frequencies->numentries; ++i) {
    fscanf(fp,"%lf",&freq);
    frequencies->contents[i] = freq;
  }
  return frequencies;
}

T
Frequencies_fill (int maxorder) {
  T frequencies;
  int order, i, j = 0;

  frequencies = Frequencies_new(maxorder);
  for (order = 0; order <= maxorder; ++order) {
    for (i = 0; i < frequencies->lengths[order]; ++i) {
      frequencies->contents[j] = markov_freqs[j];
      ++j;
    }
  }
  return frequencies;
}
  

void
Frequencies_free (T *frequencies) {
  free((*frequencies)->lengths);
  free((*frequencies)->contents);
  free(*frequencies);
}


int
Frequencies_maxorder (T frequencies) {
  return frequencies->maxorder;
}


double
Frequencies_get (T frequencies, int prob_index) {
  return frequencies->contents[prob_index];
}


void
Frequencies_normalize (T frequencies) {
  int i, j;
  double sum;

  for (i = 0; i < frequencies->numentries; i += ALPHABET_SIZE) {
    sum = 0.0;
    for (j = 0; j < ALPHABET_SIZE; ++j) {
      sum += frequencies->contents[i+j] += 0.5; /* pseudocount */
    }
    for (j = 0; j < ALPHABET_SIZE; ++j) {
      frequencies->contents[i+j] /= sum;
    }
  }
}

void
Frequencies_print (FILE *fp, T frequencies) {
  int i, j;

  for (i = 0; i < frequencies->numentries; i += ALPHABET_SIZE) {
    for (j = 0; j < ALPHABET_SIZE; ++j) {
      fprintf(fp,"%f ",frequencies->contents[i+j]);
    }
    fprintf(fp,"\n");
  }
}

